/* Laboratorio di Programmazione per Matematica e SMID, gennaio 2013 */

/* Un corso di recitazione può essere rappresentato da:
-- cognome dell'insegnante (assumendo che non ci siano omonimie e il cognome identifichi l'insegnante in modo univoco)
-- i giorni lavorativi distinti della settimana in cui si tiene (ad esempio: lunedì e giovedì; i giorni lavorativi si possono rappresentare con interi da 1 a 5 e un corso si può tenere da un minimo di 1 giorno alla settimana, a un massimo di 5 giorni alla settimana)
-- ora in cui inizia (supponendo che inizi sempre alla stessa ora - rappresentata da un intero tra 8 e 20 - in tutti i giorni della settimana in cui è previsto, e che una lezione duri 3 ore; ad esempio, una lezione che inizia alle 12 dura dalle 12 alle 15)
-- aula in cui si tiene (si supponga che le aule disponibili presso la scuola siano solo due, identificate dai numeri interi 1 e 2)

Due corsi sono incompatibili se:
1) sono tenuti dallo stesso insegnante e ci sono sovrapposizioni di orario, oppure  
2) sono tenuti nella stessa aula e ci sono sovrapposizioni di orario.

Ad esempio, due corsi che si tengono entrambi al lunedì ed iniziano uno alle 15 e l'altro alle 17 hanno una sovrapposizione di orario. Se vengono tenuti da insegnanti diversi e in aule diverse questa sovrapposizione non crea problemi e quindi i corsi sono compatibili, ma se l'insegnante o l'aula sono gli stessi, i due corsi sono incompatibili. 

Si progetti e si implementi un programma C++ che

1) chiede all'utente di inserire i dati relativi al primo corso: quando vengono richiesti i giorni in cui si tiene il corso, non è ammesso l'inserimento di un giorno già inserito; va operato un controllo sull'input in modo che i giorni siano compresi tra 1 e 5, e le ore siano comprese tra 8 e 20; 

2) stampa i dati inseriti;

3) chiede all'utente di inserire i dati relativi al secondo corso (con gli stessi controlli di cui sopra);

4) stampa i dati inseriti;

5) verifica se ci siano incompatibilità tra i due corsi, nel qual caso segnala all'utente che tipo di incompatibilità sussiste e termina. Se non ci sono incompatibilità, informa l'utente che i due corsi sono compatibili e termina.
*/

#include <string>
#include <iostream>
#include <vector>
#include <math.h>
#include <limits>   // I has to add it for g++ with WINDOWS for functions and limits in the read function
#include <stdlib.h> // I has to add it for g++ with WINDOWS for abs function (and with the .h, otherwise it does not find the header) 

using namespace std;


bool contains(vector<int> v, int day)
   {
   for (int i=0; i < v.size(); ++i)
      if (v[i] == day)
         return true; 

   return false;
   }

bool have_some_common_element(vector<int> v1, vector<int> v2)
   {
   for (int i=0; i < v1.size(); ++i)
      if (contains(v2, v1[i]))
         return true; 

   return false;

   }

class Course {
  private:
    string teacher;
    vector<int> days;
    int start_time;
    int place;
    
  public:
    Course();
    void read_data();
    void read_teacher();
    void read_days();
    void read_start_time();
    void read_place();
    void show();
    bool verify_compatibility(Course);
};

Course::Course(){
  teacher = "";
  place = 0;
  start_time = 0;
}


void Course::show(){
  cout << "\n\nIl corso del docente " << teacher << " si tiene al";
  for (int i=0; i < days.size(); ++i)
     {
     switch (days[i]) 
         {
         case 1: cout << " lunedi' ";
         break;

         case 2: cout << " martedi' ";
         break;

         case 3: cout << " mercoledi' ";
         break;

         case 4: cout << " giovedi' ";
         break;

         case 5: cout << " venerdi' ";
         break;
     }
  }
      
  cout << "dalle " << start_time << " alle " << start_time + 3 << " in aula " << place << endl << endl;
  return;
}

void Course::read_data()
{
   read_teacher();
   read_days();
   read_start_time();
   read_place();
   return;
}


void Course::read_teacher()
{
   cout << "\nInserisci il cognome dell'insegnante\n\n";    
   cin >> teacher;
   return;
}


void Course::read_days()
{
   vector<int> v;
   int day;

   cout << "\nInserisci i giorni distinti della settimana in cui si tiene il corso\n1. lunedi'\n2. martedi'\n3. mercoledi'\n4. giovedi'\n5. venerdi'\n0. termina inserimento\n\n";    

   while(!(cin>>day) || (day < 0 || day > 5))
       {
       cin.clear();
       cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
       cout << "Non hai inserito un giorno della settimana ammissibile, riprova\n"; 
       }   
   while (day != 0)
       {
        if (contains(v, day))
            cout << "Il giorno era gia' presente, riprova\n";    
        else 
            v.push_back(day);
        while(!(cin>>day) || (day < 0 || day > 5))
             {
             cin.clear();
             cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
             cout << "Non hai inserito un giorno della settimana ammissibile, riprova\n"; 
             }
         }
   days = v;
   return;
}

void Course::read_start_time()
{
   cout << "\nInserisci l'ora di inizio della lezione (compresa tra 8 e 20)\n\n";    
   while(!(cin>>start_time) || (start_time < 8 || start_time > 20))
       {
       cin.clear();
       cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
       cout << "Non hai inserito un'ora ammissibile, riprova\n";
       }
   return;
}


void Course::read_place()
{
   cout << "\nInserisci l'aula (1 oppure 2)\n\n";    
   while(!(cin>>place) || (place != 1 && place != 2))
       {
       cin.clear();
       cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
       cout << "Non hai inserito un'aula ammissibile, riprova\n";
       }
   return;
}
  

bool Course::verify_compatibility(Course c){
   if (teacher != c.teacher && place != c.place)
      {cout << "\nI corsi hanno insegnanti e aule diversi: non esistono incompatibilita'\n\n"; 
       return true;
       }
   else if (abs(start_time - c.start_time) >= 3)
           {cout << "\nI corsi condividono l'insegnante oppure l'aula ma iniziano in orari che non sollevano problemi: non esistono incompatibilita'\n\n"; 
            return true;
           }
        else if (!(have_some_common_element(days, c.days)))
             {cout << "\nI corsi condividono l'insegnante oppure l'aula e iniziano in orari che potrebbero sollere problemi, ma si tengono in giorni della settimana distinti: non esistono incompatibilita'\n\n"; 
            return true;
             }

cout << "\nI corsi sono incompatibili\n\n";           
return false;
}


int main()
{  
   Course c1 = Course();
   Course c2 = Course();
   cout << "\n************************************";
   cout << "\n* Inserimento dati del primo corso *";
   cout << "\n************************************\n";
   c1.read_data();
   c1.show();
   cout << "\n**************************************";
   cout << "\n* Inserimento dati del secondo corso *";
   cout << "\n**************************************\n";
   c2.read_data();
   c2.show();
   c1.verify_compatibility(c2);
   return 0;
}
